#include <unistd.h>

#include "subgetopt.h"

void _exit(int);

static void outs(const char *s)
{
  while (*s) {
    if (write(1, s, 1) < 0) _exit(111);
    s++;
  }
}
static void err(const char *s)
{
  while (*s) {
    if (write(2, s, 1) < 0) _exit(111);
    s++;
  }
}
static void die_usage(void)
{
  err("usage: c-auto-guarddefine [ -c ] [ -g guarddefn ] define value1 [ value2 ]\n");
  _exit(100);
}

int main(int argc, char **argv)
{
  int opt;
  char *guard_define;
  unsigned int comment_flag;

  guard_define = 0;
  comment_flag = 0;
  while ((opt = sgopt(argc, argv, "cg:")) != sgoptdone) {
    switch (opt) {
      case 'c':
        comment_flag = 1;
        break;
      case 'g':
        guard_define = sgoptarg;
        break;
      default:
        die_usage();
    }
  }
  argv += sgoptind;
  argc -= sgoptind;

  if (guard_define) {
    if (argc != 3) die_usage();
  }
  else if (comment_flag) {
    if (argc < 1) die_usage();
  }
  else {
    if (argc != 2) die_usage();
  }

  outs("/*\n  WARNING: This file was auto-generated. Do not edit!\n*/\n");

  if (comment_flag) {
    outs("/* ");
    outs(argv[0]);
    outs(" */\n");
  }
  else {
    if (guard_define) {
      outs("#ifdef ");
      outs(guard_define);
      outs("\n");
    }

    outs("#define ");
    outs(argv[0]);
    outs(" ");
    outs(argv[1]);
    outs("\n");

    if (guard_define) {
      outs("#else\n");
      outs("#define ");
      outs(argv[0]);
      outs(" ");
      outs(argv[2]);
      outs("\n");
      outs("#endif\n");
    }
  }

  _exit(0);
}
