#ifndef NEMO_CHAR_H
#define NEMO_CHAR_H

#ifdef __cplusplus
extern "C" {
#endif

/*  table of 16 hex chars */
extern const char *char_hex_chars;

/*
 table of 256 values
 if char is hex -> return binary value (0-15)
 else -> return 16;
*/
extern const byte_t *char_hex_table;


/*  table of 32 base32 chars */
extern const char *char_base32_chars;

/*
 table of 256 values
 if char is base32 -> return binary value (0-31)
 else -> return 32;
*/
extern const byte_t *char_base32_table;


/*  table of 32 base32hex chars */
extern const char *char_base32hex_chars;

/*
 table of 256 values
 if char is base32hex -> return binary value (0-31)
 else -> return 32;
*/
extern const byte_t *char_base32hex_table;


/*  table of 64 base64 chars */
extern const char *char_base64_chars;

/*
 table of 256 values
 if char is base64 -> return binary value (0-63)
 else -> return 64;
*/
extern const byte_t *char_base64_table;


/*  table of 64 base64url chars */
extern const char *char_base64url_chars;

/*
 table of 256 values
 if char is base64url -> return binary value (0-63)
 else -> return 64;
*/
extern const byte_t *char_base64url_table;


/*  table of 64 b64 chars */
extern const char *char_b64_chars;

/*
 table of 256 values
 if char is b64 -> return binary value (0-63)
 else -> return 64;
*/
extern const byte_t *char_b64_table;


extern const byte_t *char_map_table;
extern const byte_t *char_print_table;

static inline unsigned int char_isalpha(char c)
{
  return char_map_table[(byte_t)c] == '\004';
}

static inline unsigned int char_isdigit(char c)
{
  return char_map_table[(byte_t)c] == '\003';
}

static inline unsigned int char_ispunct(char c)
{
  return char_map_table[(byte_t)c] == '\002';
}

static inline unsigned int char_isspace(char c)
{
  return char_map_table[(byte_t)c] == '\001';
}

static inline unsigned int char_isgraph(char c)
{
  byte_t r;
  r = char_map_table[(byte_t)c];
  return r == '\002' || r == '\003' || r == '\004';
}

static inline unsigned int char_isprint(char c)
{
  return char_print_table[(byte_t)c];
}

extern const char *char_lower_table;
extern const char *char_upper_table;

static inline char char_tolower(char c)
{
  return char_lower_table[(byte_t)c];
}

static inline char char_toupper(char c)
{
  return char_upper_table[(byte_t)c];
}


#ifdef __cplusplus
}
#endif

#endif
