#ifndef NEMO_INT_VECTOR_H
#define NEMO_INT_VECTOR_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  int *va;
  unsigned int len;
  unsigned int a;
} int_vector;

#define INT_VECTOR { 0,0,0 }

static inline void int_vector_init(register int_vector *v)
{
  v->va = 0;
  v->a = v->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int int_vector_realloc(int_vector *v, unsigned int n);
unsigned int int_vector_alloc(int_vector *v, unsigned int n);

static inline unsigned int int_vector_ready(register int_vector *v, register unsigned int n)
{
  if (v->va) {
    if (n <= v->a) return 1;
    return int_vector_realloc(v, n);
  }
  return int_vector_alloc(v, n);
}

static inline unsigned int int_vector_readyplus(register int_vector *v, register unsigned int n)
{
  if (v->va) {
    n += v->len;
    if (n <= v->a) return 1;
    return int_vector_realloc(v, n);
  }
  return int_vector_alloc(v, n);
}

void		int_vector_free(int_vector *iv);
unsigned int	int_vector_append(int_vector *iv, const int *d);  /* beware: this takes a pointer to 1 int */
unsigned int	int_vector_erase(int_vector *iv);
void		int_vector_zero(int_vector *iv);
/*
  return length, safely
*/
static inline unsigned int int_vector_len(register const int_vector *v)
{
  if (v->va) return v->len;
  return 0;
}

/*
unsigned int	int_vector_copy(int_vector *vout, const int_vector *vfrom);
unsigned int	int_vector_cat(int_vector *vout, const int_vector *vfrom);
unsigned int	int_vector_find(const int_vector *vector, unsigned int d);
unsigned int	int_vector_rfind(const int_vector *vector, unsigned int d);
void		int_vector_remove(int_vector *vector, unsigned int i);
void		int_vector_swap(int_vector *vector, unsigned int i, unsigned int j);
void		int_vector_sort(int_vector *vector);
*/

#ifdef __cplusplus
}
#endif

#endif
