#include <unistd.h>

#include "auto_stdint.h"

void _exit(int);

static void outs(const char *s)
{
  while (*s) {
    if (write(1, s, 1) < 0) _exit(111);
    s++;
  }
}

static void outeol(void)
{
  outs("\n");
}

static void out_typedef(const char *new, const char *type)
{
  outs("typedef ");
  outs(type);
  outs(" ");
  outs(new);
  outs(";\n");
}

static void out_cond_typedef(const char *new, const char *type)
{
  outs("#ifndef ");
  outs(new);
  outeol();
  out_typedef(new, type);
  outs("#define ");
  outs(new);
  outs(" ");
  outs(type);
  outeol();
  outs("#endif\n");
}

int main(int argc, char **argv)
{
  unsigned long u;

  if (argc != 1) return 111;  /* keep compiler quiet */
  if (!*argv[0]) return 111;  /* keep compiler quiet */

  outs("#ifndef NEMO_STDINT_H\n");
  outs("#define NEMO_STDINT_H\n\n");

  outs("/*\n  WARNING: This file was auto-generated. Do not edit!\n*/\n\n");

  if (auto_stdint) {
    outs("#include <stdint.h>\n\n");
  }

  outs("#ifdef __cplusplus\n");
  outs("extern \"C\" {\n");
  outs("#endif\n\n");
/*
  Nemo LIBDJB own definition.
*/
  out_typedef("byte_t", "unsigned char");
  outeol();
/*
  test for size of long
*/
  if (!auto_stdint) {
    out_cond_typedef("int16_t", "short");
    out_cond_typedef("uint16_t", "unsigned short");
    outeol();

    u = 1;

    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;

    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;

    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;

    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;
    u += u;

    if (!u) {  /* long == 32 bit */
      out_cond_typedef("int32_t", "long");
      out_cond_typedef("uint32_t", "unsigned long");
      outeol();
      out_cond_typedef("int64_t", "long long");
      out_cond_typedef("uint64_t", "unsigned long long");
      outeol();
    }
    else {  /* long == 64 bit */
      out_cond_typedef("int32_t", "int");
      out_cond_typedef("uint32_t", "unsigned int");
      outeol();
      out_cond_typedef("int64_t", "long");
      out_cond_typedef("uint64_t", "unsigned long");
      outeol();
    }
  }

  outs("#ifdef __cplusplus\n");
  outs("}\n");
  outs("#endif\n\n");

  outs("#endif\n");

  return (0);
}
