#include "stdint.h"

#include <unistd.h>
#include <sys/stat.h>
#include <time.h>

void _exit(int);

static void outs_upper(const char *s)
{
  char x;
  while ((x = *s)) {
    if (x >= 'a' && x <= 'z') {  /* lower case */
      x = (char)(x - ('a' - 'A'));
    }
    if (write(1, &x, 1) < 0) _exit(111);
    s++;
  }
}

static void outs(const char *s)
{
  while (*s) {
    if (write(1, s, 1) < 0) _exit(111);
    s++;
  }
}

static void outeol(void)
{
  outs("\n");
}

static void out_typedef(const char *new, const char *type)
{
  outs("typedef\t");
  outs(type);
  outs("\t");
  outs(new);
  outs(";\n");
  outs("#define\t__");
  outs(new);
  outs("_defined\t1\n");
  outs("#define\t__");
  outs_upper(new);
  outs("_DEFINED\t1\n");
}

static void out_typedef_int(const char *new, unsigned int size)
{
  switch (size) {
    case sizeof(int64_t):
      out_typedef(new, "int64_t");
      break;
    case sizeof(int32_t):
      out_typedef(new, "int32_t");
      break;
    case sizeof(int16_t):
      out_typedef(new, "int16_t");
      break;
    default:
      out_typedef(new, "int");
      break;
  }
}

static void out_typedef_uint(const char *new, unsigned int size)
{
  switch (size) {
    case sizeof(uint64_t):
      out_typedef(new, "uint64_t");
      break;
    case sizeof(uint32_t):
      out_typedef(new, "uint32_t");
      break;
    case sizeof(uint16_t):
      out_typedef(new, "uint16_t");
      break;
    default:
      out_typedef(new, "unsigned int");
      break;
  }
}

int main(int argc, char **argv)
{
  if (argc != 1) return 111;  /* keep compiler quiet */
  if (!*argv[0]) return 111;  /* keep compiler quiet */

  outs("#ifndef NEMO_UNIXTYPES_H\n");
  outs("#define NEMO_UNIXTYPES_H\n\n");

  outs("/*\n  WARNING: This file was auto-generated. Do not edit!\n*/\n\n");

  outs("/*\n  common unix data types\n*/\n\n");

  outs("#ifdef __cplusplus\n");
  outs("extern \"C\" {\n");
  outs("#endif\n\n");

  out_typedef_uint("size_t", sizeof(size_t));
  outeol();
  out_typedef_int("ssize_t", sizeof(ssize_t));
  outeol();

  out_typedef_int("time_t", sizeof(time_t));
  outeol();

  out_typedef_uint("mode_t", sizeof(mode_t));
  outeol();

  out_typedef_int("pid_t", sizeof(pid_t));
  outeol();

  out_typedef_uint("uid_t", sizeof(uid_t));
  outeol();
  out_typedef_uint("gid_t", sizeof(gid_t));
  outeol();

  out_typedef_int("off_t", sizeof(off_t));
  outeol();

  outs("#ifdef __cplusplus\n");
  outs("}\n");
  outs("#endif\n\n");

  outs("#endif\n");

  return (0);
}
