#ifndef NEMO_SA_VECTOR_H
#define NEMO_SA_VECTOR_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  stralloc *va;
  unsigned int len;
  unsigned int a;
} sa_vector;

#define SA_VECTOR {0,0,0}

static inline void sa_vector_init(register sa_vector *v)
{
  v->va = 0;
  v->a = v->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int sa_vector_realloc(sa_vector *v, unsigned int n);
unsigned int sa_vector_alloc(sa_vector *v, unsigned int n);

static inline unsigned int sa_vector_ready(register sa_vector *v, register unsigned int n)
{
  if (v->va) {
    if (n <= v->a) return 1;
    return sa_vector_realloc(v, n);
  }
  return sa_vector_alloc(v, n);
}

static inline unsigned int sa_vector_readyplus(register sa_vector *v, register unsigned int n)
{
  if (v->va) {
    n += v->len;
    if (n <= v->a) return 1;
    return sa_vector_realloc(v, n);
  }
  return sa_vector_alloc(v, n);
}

unsigned int	sa_vector_cat(sa_vector *to, const sa_vector *from);
unsigned int	sa_vector_copy(sa_vector *to, const sa_vector *from);

unsigned int	sa_vector_erase(sa_vector *v);
unsigned int	sa_vector_append(sa_vector *v, const stralloc *sa);
unsigned int	sa_vector_appends(sa_vector *v, const char *s);
unsigned int	sa_vector_appendb(sa_vector *v, const void *d, unsigned int len);
unsigned int	sa_vector_insert(sa_vector *v, unsigned int pos, const stralloc *sa);

void		sa_vector_remove(sa_vector *v, unsigned int pos, unsigned int len);
void		sa_vector_purge(sa_vector *v);  /* remove all empty entries */
void		sa_vector_sort(sa_vector *v);
void		sa_vector_reverse(sa_vector *v);
void		sa_vector_swap(sa_vector *v, unsigned int pos, unsigned int pos2);

unsigned int	sa_vector_case_find(const sa_vector *v, const stralloc *target);  /* v must be sorted */
unsigned int	sa_vector_find(const sa_vector *v, const stralloc *target);  /* v must be sorted */

void		sa_vector_unique(sa_vector *v);  /* v must be sorted */

void		sa_vector_lower(sa_vector *v);

unsigned int	sa_vector_parse(sa_vector *token_list, const stralloc *tokens, const char *delimiters, unsigned int dlen);
unsigned int	sa_vector_parse_config(sa_vector *token_list, const stralloc *tokens);

unsigned int	sa_vector_0(sa_vector *v);  /* append '\0' to each entry */

void		sa_vector_zero(sa_vector *v);

void		sa_vector_free(sa_vector *v);

/*
  return length, safely
*/
static inline unsigned int sa_vector_len(register const sa_vector *v)
{
  if (v->va) return v->len;
  return 0;
}

#ifdef __cplusplus
}
#endif

#endif
