#ifndef NEMO_STRALLOC_H
#define NEMO_STRALLOC_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  char *s;
  unsigned int len;
  unsigned int a;
} stralloc;

#define STRALLOC { 0,0,0 }

static inline void stralloc_init(register stralloc *sa)
{
  sa->s = 0;
  sa->a = sa->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int stralloc_realloc(stralloc *sa, unsigned int n);
unsigned int stralloc_alloc(stralloc *sa, unsigned int n);

static inline unsigned int stralloc_ready(register stralloc *sa, register unsigned int n)
{
  if (sa->s) {
    if (n <= sa->a) return 1;
    return stralloc_realloc(sa, n);
  }
  return stralloc_alloc(sa, n);
}

static inline unsigned int stralloc_readyplus(register stralloc *sa, register unsigned int n)
{
  if (sa->s) {
    n += sa->len;
    if (n <= sa->a) return 1;
    return stralloc_realloc(sa, n);
  }
  return stralloc_alloc(sa, n);
}

void		stralloc_free(stralloc *sa);

unsigned int	stralloc_copy(stralloc *sato, const stralloc *safrom);
unsigned int	stralloc_cat(stralloc *sato, const stralloc *safrom);
unsigned int	stralloc_copys(stralloc *sa, const char *s);
unsigned int	stralloc_cats(stralloc *sa, const char *s);
unsigned int	stralloc_copyb(stralloc *sa, const void *d, unsigned int dlen);
unsigned int	stralloc_catb(stralloc *sa, const void *d, unsigned int dlen);
unsigned int	stralloc_cathexb(stralloc *sa, const void *d, unsigned int dlen);

unsigned int	stralloc_append(stralloc *sa, const void *c);

unsigned int	stralloc_catlong0(stralloc *sa, long l, unsigned int n);
unsigned int	stralloc_catulong0(stralloc *sa, unsigned long u, unsigned int n);

unsigned int	stralloc_catlong(stralloc *sa, long l);
unsigned int	stralloc_catulong(stralloc *sa, unsigned long ul);
unsigned int	stralloc_catxlong(stralloc *sa, unsigned long l);

unsigned int	stralloc_catint(stralloc *sa, int i);
unsigned int	stralloc_catint0(stralloc *sa, int i, unsigned int n);

unsigned int	stralloc_catuint(stralloc *sa, unsigned int ui);
unsigned int	stralloc_catuint0(stralloc *sa, unsigned int ui, unsigned int n);

unsigned int	stralloc_erase(stralloc *sa);		/* erase contents */
unsigned int	stralloc_0(stralloc *sa);		/* append '\0' to sa */

/*  return 1: s2 > s1;  0: s1 == s2;  -1: s1 > s2  */
int		stralloc_case_diff(const stralloc *sa1, const stralloc *sa2);
int		stralloc_case_diffs(const stralloc *sa, const char *s);
int		stralloc_diff(const stralloc *sa1, const stralloc *sa2);
int		stralloc_diffb(const stralloc *sa1, const void *d, unsigned int len);
int		stralloc_diffs(const stralloc *sa1, const char *s);

/*  return 1=true, 0=false  */
unsigned int	stralloc_start(const stralloc *sa, const stralloc *prefix);
unsigned int	stralloc_startb(const stralloc *sa, const void *prefix, unsigned int len);
unsigned int	stralloc_starts(const stralloc *sa, const char *prefix);
unsigned int	stralloc_end(const stralloc *sa, const stralloc *suffix);
unsigned int	stralloc_endb(const stralloc *sa, const void *suffix, unsigned int len);
unsigned int	stralloc_ends(const stralloc *sa, const char *suffix);
unsigned int	stralloc_equal(const stralloc *sa1, const stralloc *sa2);
unsigned int	stralloc_equalb(const stralloc *sa, const void *d, unsigned int dlen);
unsigned int	stralloc_equals(const stralloc *sa, const char *s);
unsigned int	stralloc_case_start(const stralloc *sa, const stralloc *prefix);
unsigned int	stralloc_case_starts(const stralloc *sa, const char *prefix);
unsigned int	stralloc_case_equal(const stralloc *sa1, const stralloc *sa2);
unsigned int	stralloc_case_equals(const stralloc *sa, const char *s);
unsigned int	stralloc_case_end(const stralloc *sa, const stralloc *suffix);
unsigned int	stralloc_case_ends(const stralloc *sa, const char *suffix);

/*  return index to found position, sa->len = not found  */
unsigned int	stralloc_chr(const stralloc *sa, char c);
unsigned int	stralloc_rchr(const stralloc *sa, char c);

/*  return count of c in sa  */
unsigned int	stralloc_count(const stralloc *sa, char c);

/*  return index to first occurance of data, sa->len = not found  */
unsigned int    stralloc_find(const stralloc *sa, const stralloc *what);
unsigned int    stralloc_findb(const stralloc *sa, const void *what, unsigned int len);
unsigned int    stralloc_finds(const stralloc *sa, const char *what);

void		stralloc_zero(stralloc *sa);
void		stralloc_replace(stralloc *sa, char before, char after);
void		stralloc_replace_non_printable(stralloc *sa, char ch);

/*
  return length, safely
*/
static inline unsigned int stralloc_len(register const stralloc *sa)
{
  if (sa->s) return sa->len;
  return 0;
}
/*
  case change
*/
void	stralloc_lower(stralloc *sa);
void	stralloc_upper(stralloc *sa);
/*
  remove len chars at pos (safely)
*/
void	stralloc_remove(stralloc *sa, unsigned int pos, unsigned int len);
/*
  remove all listed chars (safely)
*/
void	stralloc_remove_all(stralloc *sa, const char *delchars, unsigned int delcharslen);
/*
  trim trailing chars
*/
void	stralloc_trim(stralloc *sa, const char *trimchars, unsigned int trimcharslen);
/*
  1 --> OK
  0 --> memory exhausted, invalid data (check errno)
*/
unsigned int	stralloc_hex_encode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base32_encode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base32hex_encode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base64_encode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base64url_encode(stralloc *out, const stralloc *in);
unsigned int	stralloc_qp_encode(stralloc *out, const stralloc *in);

unsigned int	stralloc_hex_decode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base32_decode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base32hex_decode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base64_decode(stralloc *out, const stralloc *in);
unsigned int	stralloc_base64url_decode(stralloc *out, const stralloc *in);
unsigned int	stralloc_qp_decode(stralloc *out, const stralloc *in);

#ifdef __cplusplus
}
#endif

#endif
