#include "stdint.h"
#include "unixtypes.h"
#include "unix.h"
#include "stralloc.h"
#include "macro_unused.h"

void _exit(int);

static unsigned int errors = 0;

static void outs(const char *s)
{
  while (*s) {
    if (write(1, s, 1) < 0) _exit(111);
    s++;
  }
}
static void outsa(const stralloc *sa)
{
  if (write(1, sa->s, sa->len) < 0) _exit(1);
}

static void die_nomem(void)
{
  outs("fatal: out of memory\n");
  _exit(1);
}

static void err_compare(const char *in, const char *expected, const stralloc *result)
{
  errors++;
  outs("in='");
  outs(in);
  outs("'\n");
  outs("expected='");
  outs(expected);
  outs("'\n");
  outs("result='");
  outsa(result);
  outs("'\n");
}

static stralloc d1 = STRALLOC;
static stralloc d2 = STRALLOC;

static void test_upper(const char *in, const char *expected)
{
  if (!stralloc_copys(&d1, in)) die_nomem();
  stralloc_upper(&d1);
  if (!stralloc_copys(&d2, expected)) die_nomem();
  if (!stralloc_equal(&d1, &d2)) {
    err_compare(in, expected, &d1);
  }
}

static void test_lower(const char *in, const char *expected)
{
  if (!stralloc_copys(&d1, in)) die_nomem();
  stralloc_lower(&d1);
  if (!stralloc_copys(&d2, expected)) die_nomem();
  if (!stralloc_equal(&d1, &d2)) {
    err_compare(in, expected, &d1);
  }
}

int main(int argc __UNUSED__, char **argv __UNUSED__)
{
  test_lower("./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", "./0123456789abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz");
  test_upper("./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ");

  if (errors) {
    outs("test_stralloc_caseshift failed tests!\n");
    _exit(2);
  }
  _exit(0);
}
