#include "stdint.h"
#include "uint32_vector.h"
#include "error.h"

unsigned int uint32_vector_utf8_decode(register uint32_vector *v, const void *in, unsigned int len)
{
  uint32_t t;
  unsigned int flag_cont;
  unsigned int i;
  const byte_t *s;
  register byte_t ch;

  s = in;
  flag_cont = 0;
  if (!uint32_vector_erase(v)) return 0;
  for (i = 0; i < len; i++) {
    ch = *s++;
    if (flag_cont) {
      if ((ch & 0xc0) != 0x80) {  /* error, should we abort? */
        if (!uint32_vector_append(v, t)) return 0;  /* save partial */
	if (!uint32_vector_append(v, ch)) return 0;
	flag_cont = 0;
	continue;
      }
      flag_cont--;
      t <<= 6;
      t |= ch & 0x3f;
      if (!flag_cont) {
        if (!uint32_vector_append(v, t)) return 0;  /* complete */
      }
      continue;
    }
    if ((ch & 0x80) != 0x80) {  /* 1 byte, U+0000 - U+007F */
      if (!uint32_vector_append(v, ch)) return 0;
      flag_cont = 0;  /* 0 continuation bytes */
      continue;
    }
    if ((ch & 0xe0) == 0xc0) {  /* 2 byte, U+0080 - U+07FF */
      t = ch & 0x1f;
      flag_cont = 1;  /* 1 continuation byte */
      continue;
    }
    if ((ch & 0xf0) == 0xe0) {  /* 3 byte, U+0800 - U+FFFF */
      t = ch & 0x0f;
      flag_cont = 2;  /* 2 continuation bytes */
      continue;
    }
    if ((ch & 0xf8) == 0xf0) {  /* 4 bytes, U+10000 - U+10FFFF */
      t = ch & 0x07;
      flag_cont = 3;  /* 3 continuation bytes */
      continue;
    }
    if (!uint32_vector_append(v, ch)) return 0;  /* save partial */
  }

  return 1;
}
