#ifndef NEMO_UINT64_VECTOR_H
#define NEMO_UINT64_VECTOR_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  uint64_t *va;
  unsigned int len;
  unsigned int a;
} uint64_vector;

#define UINT64_VECTOR { 0,0,0 }

static inline void uint64_vector_init(register uint64_vector *v)
{
  v->va = 0;
  v->a = v->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int uint64_vector_realloc(uint64_vector *v, unsigned int n);
unsigned int uint64_vector_alloc(uint64_vector *v, unsigned int n);

static inline unsigned int uint64_vector_ready(register uint64_vector *v, register unsigned int n)
{
  if (v->va) {
    if (n <= v->a) return 1;
    return uint64_vector_realloc(v, n);
  }
  return uint64_vector_alloc(v, n);
}

static inline unsigned int uint64_vector_readyplus(register uint64_vector *v, register unsigned int n)
{
  if (v->va) {
    n += v->len;
    if (n <= v->a) return 1;
    return uint64_vector_realloc(v, n);
  }
  return uint64_vector_alloc(v, n);
}

unsigned int	uint64_vector_copy(uint64_vector *vout, const uint64_vector *vfrom);
unsigned int	uint64_vector_cat(uint64_vector *vout, const uint64_vector *vfrom);

unsigned int	uint64_vector_append(uint64_vector *v, uint64_t d);

unsigned int	uint64_vector_erase(uint64_vector *v);	/* erase contents */
/*
  return index to found position, uiv->len = not found
*/
unsigned int	uint64_vector_find(const uint64_vector *v, uint64_t d);
unsigned int	uint64_vector_rfind(const uint64_vector *v, uint64_t d);
/*
  return length, safely
*/
static inline unsigned int uint64_vector_len(register const uint64_vector *v)
{
  if (v->va) return v->len;
  return 0;
}
/*
  vector manipulation
*/
void	uint64_vector_remove(uint64_vector *v, unsigned int i);
void	uint64_vector_swap(uint64_vector *v, unsigned int i, unsigned int j);
void	uint64_vector_sort(uint64_vector *v);

void	uint64_vector_init(uint64_vector *v);
void	uint64_vector_free(uint64_vector *v);
/*
  overwrite with zeroes all allocated space (security - being very paranoid)
*/
void	uint64_vector_zero(uint64_vector *v);

#ifdef __cplusplus
}
#endif

#endif
