#ifndef NEMO_UINT_VECTOR_H
#define NEMO_UINT_VECTOR_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  unsigned int *va;
  unsigned int len;
  unsigned int a;
} uint_vector;

#define UINT_VECTOR { 0,0,0 }

static inline void uint_vector_init(register uint_vector *v)
{
  v->va = 0;
  v->a = v->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int uint_vector_realloc(uint_vector *v, unsigned int n);
unsigned int uint_vector_alloc(uint_vector *v, unsigned int n);

static inline unsigned int uint_vector_ready(register uint_vector *v, register unsigned int n)
{
  if (v->va) {
    if (n <= v->a) return 1;
    return uint_vector_realloc(v, n);
  }
  return uint_vector_alloc(v, n);
}

static inline unsigned int uint_vector_readyplus(register uint_vector *v, register unsigned int n)
{
  if (v->va) {
    n += v->len;
    if (n <= v->a) return 1;
    return uint_vector_realloc(v, n);
  }
  return uint_vector_alloc(v, n);
}

unsigned int	uint_vector_copy(uint_vector *vout, const uint_vector *vfrom);
unsigned int	uint_vector_cat(uint_vector *vout, const uint_vector *vfrom);

unsigned int	uint_vector_append(uint_vector *vector, unsigned int d);

unsigned int	uint_vector_erase(uint_vector *vector);	/* erase contents */
/*
  return index to found position, uiv->len = not found
*/
unsigned int	uint_vector_find(const uint_vector *vector, unsigned int d);
unsigned int	uint_vector_rfind(const uint_vector *vector, unsigned int d);
/*
  return length, safely
*/
static inline unsigned int uint_vector_len(register const uint_vector *v)
{
  if (v->va) return v->len;
  return 0;
}
/*
  vector manipulation
*/
void	uint_vector_remove(uint_vector *vector, unsigned int i);
void	uint_vector_swap(uint_vector *vector, unsigned int i, unsigned int j);
void	uint_vector_sort(uint_vector *vector);

void	uint_vector_free(uint_vector *vector);
/*
  overwrite with zeroes all allocated space (security - being very paranoid)
*/
void	uint_vector_zero(uint_vector *vector);

#ifdef __cplusplus
}
#endif

#endif
